/*     */ package org.hibernate.cfg.annotations;
/*     */ 
/*     */ import java.util.ArrayList;
/*     */ import java.util.HashMap;
/*     */ import java.util.HashSet;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Map.Entry;
/*     */ import java.util.Set;
/*     */ import javax.persistence.ColumnResult;
/*     */ import javax.persistence.EntityResult;
/*     */ import javax.persistence.FieldResult;
/*     */ import javax.persistence.SqlResultSetMapping;
/*     */ import org.apache.commons.logging.Log;
/*     */ import org.apache.commons.logging.LogFactory;
/*     */ import org.hibernate.LockMode;
/*     */ import org.hibernate.MappingException;
/*     */ import org.hibernate.cfg.BinderHelper;
/*     */ import org.hibernate.cfg.ExtendedMappings;
/*     */ import org.hibernate.cfg.QuerySecondPass;
/*     */ import org.hibernate.engine.ResultSetMappingDefinition;
/*     */ import org.hibernate.engine.query.sql.NativeSQLQueryRootReturn;
/*     */ import org.hibernate.engine.query.sql.NativeSQLQueryScalarReturn;
/*     */ import org.hibernate.mapping.Component;
/*     */ import org.hibernate.mapping.PersistentClass;
/*     */ import org.hibernate.mapping.Property;
/*     */ import org.hibernate.mapping.ToOne;
/*     */ import org.hibernate.mapping.Value;
/*     */ import org.hibernate.util.CollectionHelper;
/*     */ import org.hibernate.util.StringHelper;
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ResultsetMappingSecondPass
/*     */   implements QuerySecondPass
/*     */ {
/*  38 */   private static Log log = LogFactory.getLog(ResultsetMappingSecondPass.class);
/*     */   private SqlResultSetMapping ann;
/*     */   private ExtendedMappings mappings;
/*     */   private boolean isDefault;
/*     */   
/*     */   public ResultsetMappingSecondPass(SqlResultSetMapping ann, ExtendedMappings mappings, boolean isDefault) {
/*  44 */     this.ann = ann;
/*  45 */     this.mappings = mappings;
/*  46 */     this.isDefault = isDefault;
/*     */   }
/*     */   
/*     */   public void doSecondPass(Map persistentClasses) throws MappingException
/*     */   {
/*  51 */     if (this.ann == null) return;
/*  52 */     ResultSetMappingDefinition definition = new ResultSetMappingDefinition(this.ann.name());
/*  53 */     if (log.isInfoEnabled()) { log.info("Binding resultset mapping: " + definition.getName());
/*     */     }
/*  55 */     int entityAliasIndex = 0;
/*     */     
/*  57 */     for (EntityResult entity : this.ann.entities())
/*     */     {
/*  59 */       List properties = new ArrayList();
/*  60 */       List propertyNames = new ArrayList();
/*  61 */       Map propertyresults = new HashMap();
/*  62 */       for (FieldResult field : entity.fields())
/*     */       {
/*  64 */         String name = field.name();
/*  65 */         if (name.indexOf('.') == -1)
/*     */         {
/*  67 */           properties.add(field);
/*  68 */           propertyNames.add(name);
/*     */ 
/*     */ 
/*     */ 
/*     */         }
/*     */         else
/*     */         {
/*     */ 
/*     */ 
/*  77 */           PersistentClass pc = this.mappings.getClass(entity.entityClass().getName());
/*  78 */           if (pc == null) {
/*  79 */             throw new MappingException("Entity not found " + entity.entityClass().getName() + " in SqlResultsetMapping " + this.ann.name());
/*     */           }
/*     */           
/*     */ 
/*     */ 
/*  84 */           int dotIndex = name.lastIndexOf('.');
/*  85 */           String reducedName = name.substring(0, dotIndex);
/*  86 */           Iterator parentPropIter = getSubPropertyIterator(pc, reducedName);
/*  87 */           List followers = getFollowers(parentPropIter, reducedName, name);
/*     */           
/*  89 */           int index = propertyNames.size();
/*  90 */           int followersSize = followers.size();
/*  91 */           for (int loop = 0; loop < followersSize; loop++) {
/*  92 */             String follower = (String)followers.get(loop);
/*  93 */             int currentIndex = getIndexOfFirstMatchingProperty(propertyNames, follower);
/*  94 */             index = (currentIndex != -1) && (currentIndex < index) ? currentIndex : index;
/*     */           }
/*  96 */           propertyNames.add(index, name);
/*  97 */           properties.add(index, field);
/*     */         }
/*     */       }
/*     */       
/* 101 */       Set uniqueReturnProperty = new HashSet();
/* 102 */       Iterator iterator = properties.iterator();
/* 103 */       while (iterator.hasNext()) {
/* 104 */         FieldResult propertyresult = (FieldResult)iterator.next();
/* 105 */         String name = propertyresult.name();
/* 106 */         if ("class".equals(name)) {
/* 107 */           throw new MappingException("class is not a valid property name to use in a @FieldResult, use @Entity(discriminatorColumn) instead");
/*     */         }
/*     */         
/*     */ 
/* 111 */         ArrayList allResultColumns = new ArrayList();
/* 112 */         allResultColumns.add(propertyresult.column());
/*     */         
/* 114 */         if (uniqueReturnProperty.contains(name)) {
/* 115 */           throw new MappingException("duplicate @FieldResult for property " + name + " on @Entity " + entity.entityClass().getName() + " in " + this.ann.name());
/*     */         }
/*     */         
/*     */ 
/*     */ 
/* 120 */         uniqueReturnProperty.add(name);
/* 121 */         String key = StringHelper.root(name);
/* 122 */         ArrayList intermediateResults = (ArrayList)propertyresults.get(key);
/* 123 */         if (intermediateResults == null) {
/* 124 */           propertyresults.put(key, allResultColumns);
/*     */         }
/*     */         else {
/* 127 */           intermediateResults.addAll(allResultColumns);
/*     */         }
/*     */       }
/* 130 */       Iterator entries = propertyresults.entrySet().iterator();
/* 131 */       while (entries.hasNext()) {
/* 132 */         Map.Entry entry = (Map.Entry)entries.next();
/* 133 */         if ((entry.getValue() instanceof ArrayList)) {
/* 134 */           ArrayList list = (ArrayList)entry.getValue();
/* 135 */           entry.setValue(list.toArray(new String[list.size()]));
/*     */         }
/*     */       }
/*     */       
/* 139 */       if (!BinderHelper.isDefault(entity.discriminatorColumn())) {
/* 140 */         propertyresults.put("class", new String[] { entity.discriminatorColumn() });
/*     */       }
/*     */       
/* 143 */       propertyresults = propertyresults.isEmpty() ? CollectionHelper.EMPTY_MAP : propertyresults;
/* 144 */       NativeSQLQueryRootReturn result = new NativeSQLQueryRootReturn("alias" + entityAliasIndex++, entity.entityClass().getName(), propertyresults, LockMode.READ);
/*     */       
/*     */ 
/*     */ 
/* 148 */       definition.addQueryReturn(result);
/*     */     }
/*     */     
/* 151 */     for (ColumnResult column : this.ann.columns()) {
/* 152 */       definition.addQueryReturn(new NativeSQLQueryScalarReturn(column.name(), null));
/*     */     }
/*     */     
/* 155 */     if (this.isDefault) {
/* 156 */       this.mappings.addDefaultResultSetMapping(definition);
/*     */     }
/*     */     else {
/* 159 */       this.mappings.addResultSetMapping(definition);
/*     */     }
/*     */   }
/*     */   
/*     */   private List getFollowers(Iterator parentPropIter, String reducedName, String name) {
/* 164 */     boolean hasFollowers = false;
/* 165 */     List followers = new ArrayList();
/* 166 */     while (parentPropIter.hasNext()) {
/* 167 */       String currentPropertyName = ((Property)parentPropIter.next()).getName();
/* 168 */       String currentName = reducedName + '.' + currentPropertyName;
/* 169 */       if (hasFollowers) {
/* 170 */         followers.add(currentName);
/*     */       }
/* 172 */       if (name.equals(currentName)) hasFollowers = true;
/*     */     }
/* 174 */     return followers;
/*     */   }
/*     */   
/*     */   private Iterator getSubPropertyIterator(PersistentClass pc, String reducedName) {
/* 178 */     Value value = pc.getRecursiveProperty(reducedName).getValue();
/*     */     Iterator parentPropIter;
/* 180 */     if ((value instanceof Component)) {
/* 181 */       Component comp = (Component)value;
/* 182 */       parentPropIter = comp.getPropertyIterator();
/*     */     }
/* 184 */     else if ((value instanceof ToOne)) {
/* 185 */       ToOne toOne = (ToOne)value;
/* 186 */       PersistentClass referencedPc = this.mappings.getClass(toOne.getReferencedEntityName());
/* 187 */       Iterator parentPropIter; if (toOne.getReferencedPropertyName() != null) {
/*     */         try {
/* 189 */           parentPropIter = ((Component)referencedPc.getRecursiveProperty(toOne.getReferencedPropertyName()).getValue()).getPropertyIterator();
/*     */ 
/*     */         }
/*     */         catch (ClassCastException e)
/*     */         {
/* 194 */           throw new MappingException("dotted notation reference neither a component nor a many/one to one", e);
/*     */         }
/*     */         
/*     */       }
/*     */       else {
/*     */         try
/*     */         {
/* 201 */           if (referencedPc.getIdentifierMapper() == null) {
/* 202 */             parentPropIter = ((Component)referencedPc.getIdentifierProperty().getValue()).getPropertyIterator();
/*     */           }
/*     */           else
/*     */           {
/* 206 */             parentPropIter = referencedPc.getIdentifierMapper().getPropertyIterator();
/*     */           }
/*     */         } catch (ClassCastException e) {
/*     */           Iterator parentPropIter;
/* 210 */           throw new MappingException("dotted notation reference neither a component nor a many/one to one", e);
/*     */         }
/*     */         
/*     */       }
/*     */     }
/*     */     else
/*     */     {
/* 217 */       throw new MappingException("dotted notation reference neither a component nor a many/one to one"); }
/*     */     Iterator parentPropIter;
/* 219 */     return parentPropIter;
/*     */   }
/*     */   
/*     */   private static int getIndexOfFirstMatchingProperty(List propertyNames, String follower) {
/* 223 */     int propertySize = propertyNames.size();
/* 224 */     for (int propIndex = 0; propIndex < propertySize; propIndex++) {
/* 225 */       if (((String)propertyNames.get(propIndex)).startsWith(follower)) {
/* 226 */         return propIndex;
/*     */       }
/*     */     }
/* 229 */     return -1;
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\cfg\annotations\ResultsetMappingSecondPass.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */