/*     */ package org.hibernate.ejb.event;
/*     */ 
/*     */ import java.lang.annotation.Annotation;
/*     */ import java.lang.annotation.ElementType;
/*     */ import java.lang.annotation.Target;
/*     */ import java.lang.reflect.Method;
/*     */ import java.util.ArrayList;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import javax.persistence.Entity;
/*     */ import javax.persistence.EntityListeners;
/*     */ import javax.persistence.ExcludeDefaultListeners;
/*     */ import javax.persistence.ExcludeSuperclassListeners;
/*     */ import javax.persistence.MappedSuperclass;
/*     */ import javax.persistence.PersistenceException;
/*     */ import org.hibernate.annotations.common.reflection.ReflectionManager;
/*     */ import org.hibernate.annotations.common.reflection.XClass;
/*     */ import org.hibernate.annotations.common.reflection.XMethod;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public final class CallbackResolver
/*     */ {
/*  35 */   private static boolean useAnnotationAnnotatedByListener = false;
/*  36 */   static { Target target = (Target)EntityListeners.class.getAnnotation(Target.class);
/*  37 */     if (target != null) {
/*  38 */       for (ElementType type : target.value()) {
/*  39 */         if (type.equals(ElementType.ANNOTATION_TYPE)) { useAnnotationAnnotatedByListener = true;
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public static Callback[] resolveCallback(XClass beanClass, Class annotation, ReflectionManager reflectionManager)
/*     */   {
/*  48 */     List<Callback> callbacks = new ArrayList();
/*  49 */     List<String> callbacksMethodNames = new ArrayList();
/*  50 */     List<Class> orderedListeners = new ArrayList();
/*  51 */     XClass currentClazz = beanClass;
/*  52 */     boolean stopListeners = false;
/*  53 */     boolean stopDefaultListeners = false;
/*     */     do {
/*  55 */       Callback callback = null;
/*  56 */       List<XMethod> methods = currentClazz.getDeclaredMethods();
/*  57 */       int size = methods.size();
/*  58 */       for (int i = 0; i < size; i++) {
/*  59 */         XMethod xMethod = (XMethod)methods.get(i);
/*  60 */         if (xMethod.isAnnotationPresent(annotation)) {
/*  61 */           Method method = reflectionManager.toMethod(xMethod);
/*  62 */           String methodName = method.getName();
/*  63 */           if (!callbacksMethodNames.contains(methodName))
/*     */           {
/*  65 */             if (callback == null) {
/*  66 */               callback = new BeanCallback(method);
/*  67 */               Class returnType = method.getReturnType();
/*  68 */               Class[] args = method.getParameterTypes();
/*  69 */               if ((returnType != Void.TYPE) || (args.length != 0)) {
/*  70 */                 throw new RuntimeException("Callback methods annotated on the bean class must return void and take no arguments: " + annotation.getName() + " - " + xMethod);
/*     */               }
/*     */               
/*     */ 
/*     */ 
/*  75 */               if (!method.isAccessible()) {
/*  76 */                 method.setAccessible(true);
/*     */               }
/*  78 */               callbacks.add(0, callback);
/*  79 */               callbacksMethodNames.add(0, methodName);
/*     */             }
/*     */             else {
/*  82 */               throw new PersistenceException("You can only annotate one callback method with " + annotation.getName() + " in bean class: " + beanClass.getName());
/*     */             }
/*     */           }
/*     */         }
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*  90 */       if (!stopListeners) {
/*  91 */         getListeners(currentClazz, orderedListeners);
/*  92 */         stopListeners = currentClazz.isAnnotationPresent(ExcludeSuperclassListeners.class);
/*  93 */         stopDefaultListeners = currentClazz.isAnnotationPresent(ExcludeDefaultListeners.class);
/*     */       }
/*     */       do
/*     */       {
/*  97 */         currentClazz = currentClazz.getSuperclass();
/*     */ 
/*     */       }
/* 100 */       while ((currentClazz != null) && (!currentClazz.isAnnotationPresent(Entity.class)) && (!currentClazz.isAnnotationPresent(MappedSuperclass.class)));
/*     */ 
/*     */ 
/*     */     }
/* 104 */     while (currentClazz != null);
/*     */     
/*     */ 
/* 107 */     if (!stopDefaultListeners) {
/* 108 */       List<Class> defaultListeners = (List)reflectionManager.getDefaults().get(EntityListeners.class);
/*     */       
/* 110 */       if (defaultListeners != null) {
/* 111 */         int defaultListenerSize = defaultListeners.size();
/* 112 */         for (int i = defaultListenerSize - 1; i >= 0; i--) {
/* 113 */           orderedListeners.add(defaultListeners.get(i));
/*     */         }
/*     */       }
/*     */     }
/*     */     
/* 118 */     for (Class listener : orderedListeners) {
/* 119 */       Callback callback = null;
/* 120 */       if (listener != null) {
/* 121 */         XClass xListener = reflectionManager.toXClass(listener);
/* 122 */         callbacksMethodNames = new ArrayList();
/*     */         do {
/* 124 */           List<XMethod> methods = xListener.getDeclaredMethods();
/* 125 */           int size = methods.size();
/* 126 */           for (int i = 0; i < size; i++) {
/* 127 */             XMethod xMethod = (XMethod)methods.get(i);
/* 128 */             if (xMethod.isAnnotationPresent(annotation)) {
/* 129 */               Method method = reflectionManager.toMethod(xMethod);
/* 130 */               String methodName = method.getName();
/* 131 */               if (!callbacksMethodNames.contains(methodName))
/*     */               {
/* 133 */                 if (callback == null) {
/*     */                   try {
/* 135 */                     callback = new ListenerCallback(method, listener.newInstance());
/*     */                   }
/*     */                   catch (IllegalAccessException e) {
/* 138 */                     throw new PersistenceException("Unable to create instance of " + listener.getName() + " as a listener of beanClass", e);
/*     */ 
/*     */                   }
/*     */                   catch (InstantiationException e)
/*     */                   {
/*     */ 
/* 144 */                     throw new PersistenceException("Unable to create instance of " + listener.getName() + " as a listener of beanClass", e);
/*     */                   }
/*     */                   
/*     */ 
/*     */ 
/* 149 */                   Class returnType = method.getReturnType();
/* 150 */                   Class[] args = method.getParameterTypes();
/* 151 */                   if ((returnType != Void.TYPE) || (args.length != 1)) {
/* 152 */                     throw new PersistenceException("Callback methods annotated in a listener bean class must return void and take one argument: " + annotation.getName() + " - " + method);
/*     */                   }
/*     */                   
/*     */ 
/*     */ 
/* 157 */                   if (!method.isAccessible()) {
/* 158 */                     method.setAccessible(true);
/*     */                   }
/* 160 */                   callbacks.add(0, callback);
/*     */                 }
/*     */                 else {
/* 163 */                   throw new PersistenceException("You can only annotate one callback method with " + annotation.getName() + " in bean class: " + beanClass.getName() + " and callback listener: " + listener.getName());
/*     */                 }
/*     */               }
/*     */             }
/*     */           }
/*     */           
/*     */ 
/*     */ 
/*     */ 
/* 172 */           xListener = null;
/*     */         }
/* 174 */         while (xListener != null);
/*     */       }
/*     */     }
/* 177 */     return (Callback[])callbacks.toArray(new Callback[callbacks.size()]);
/*     */   }
/*     */   
/*     */   private static void getListeners(XClass currentClazz, List<Class> orderedListeners) {
/* 181 */     EntityListeners entityListeners = (EntityListeners)currentClazz.getAnnotation(EntityListeners.class);
/* 182 */     if (entityListeners != null) {
/* 183 */       Class[] classes = entityListeners.value();
/* 184 */       int size = classes.length;
/* 185 */       for (int index = size - 1; index >= 0; index--) {
/* 186 */         orderedListeners.add(classes[index]);
/*     */       }
/*     */     }
/* 189 */     if (useAnnotationAnnotatedByListener) {
/* 190 */       Annotation[] annotations = currentClazz.getAnnotations();
/* 191 */       for (Annotation annot : annotations) {
/* 192 */         entityListeners = (EntityListeners)annot.getClass().getAnnotation(EntityListeners.class);
/* 193 */         if (entityListeners != null) {
/* 194 */           Class[] classes = entityListeners.value();
/* 195 */           int size = classes.length;
/* 196 */           for (int index = size - 1; index >= 0; index--) {
/* 197 */             orderedListeners.add(classes[index]);
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-entitymanager-3.3.2.GA.jar!\org\hibernate\ejb\event\CallbackResolver.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */